// ////////////////////////////////////////////////////
//
// StdView.cpp : implementation of the CStdView class
// Based on class COGLView : public CView
// Inspect COGLView for standard OpenGL initialization
//
////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Std.h"

#include "StdDoc.h"
#include "StdView.h"
#include "math.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////
// You can put definitions, variables and helperfunctions
// here. Functions that does not rely on CStdView member
// variables or functions

#define PI  3.14159265359f
#define MY_TIMER 4
#define TURNS_PR_UNITLENGTH 2.0f


//////////////////////////////////////////////////////////
// CStdView

IMPLEMENT_DYNCREATE(CStdView, COGLView)

BEGIN_MESSAGE_MAP(CStdView, COGLView)
	//{{AFX_MSG_MAP(CStdView)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_COMMAND(ID_ANIM_BOLT, OnAnimBolt)
	ON_COMMAND(ID_ANIM_NUT, OnAnimNut)
	ON_UPDATE_COMMAND_UI(ID_ANIM_BOLT, OnUpdateAnimBolt)
	ON_UPDATE_COMMAND_UI(ID_ANIM_NUT, OnUpdateAnimNut)
	ON_WM_TIMER()
	ON_COMMAND(ID_ANIM_RESET, OnAnimReset)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

///////////////////////////////////////////////////////////
// CStdView construction/destruction

CStdView::CStdView()
{
	m_xv=-60.0f;
	m_yv=0.0f;
	aniBolt=FALSE;
	aniNut=FALSE;
	m_boltv=m_nutv=0.0f;

	m_boltOffset=m_nutOffset=0.0f;

	m_hRC=NULL;
	m_hDC=NULL;

};

CStdView::~CStdView()
{
}

////////////////////////////////////////////////////////////
// CStdView diagnostics
// You should usually not touch these:
#ifdef _DEBUG
void CStdView::AssertValid() const
{COGLView::AssertValid();}
void CStdView::Dump(CDumpContext& dc) const
{COGLView::Dump(dc);}

CStdDoc* CStdView::GetDocument() // non-debug version is inline
{	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CStdDoc)));
	return (CStdDoc*)m_pDocument;
}
#endif //_DEBUG

//////////////////////////////////////////////////////////////
// CStdView operations

void CStdView::DrawScene()
{
	// Clear the color and depth buffers.
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		
	// Set up for scene
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	//////////////////////////////////
	// bring construction in position

	// Based on:	default eye is at 0,0,0, 
	//				looking twds negative z-axis

	// move the construction down z
	glTranslatef(0.0f,0.0f,-18.0f);
	// rotate around x-axis 
	glRotatef(m_xv,1.0f,0.0f,0.0f);
	// rotate around z-axis 
	glRotatef(m_yv,0.0f,0.0f,1.0f);

	/////////////////////////////
	// draw Coordinate system
	/*
	GLUquadricObj *glpQ;
	glpQ=gluNewQuadric();
	glLineWidth(1.0f);
	CMaterial::SetMaterial(BLACK_PLASTIC);
	glBegin(GL_LINES);
	glVertex3f(-3.0f,0.0f,0.0f);
	glVertex3f(5.0f,0.0f,0.0f);
	glVertex3f(0.0f,-3.0f,0.0f);
	glVertex3f(0.0f,5.0f,0.0f);
	glVertex3f(0.0f,0.0f,-2.0f);
	glVertex3f(0.0f,0.0f,10.0f);
	glEnd();
	glPushMatrix();
		glTranslatef(5.0f,0.0f,0.0f);
		glRotatef(90.0f,0.0f,1.0f,0.0f);
		gluCylinder(glpQ,0.3f,0.0f,0.5f,10,10);
	glPopMatrix();
	glPushMatrix();
		glTranslatef(0.0f,5.0f,0.0f);
		glRotatef(-90.0f,1.0f,0.0f,0.0f);
		gluCylinder(glpQ,0.3f,0.0f,0.5f,10,10);
	glPopMatrix();
	glPushMatrix();
		glTranslatef(0.0f,0.0f,10.0f);
		gluCylinder(glpQ,0.3f,0.0f,0.5f,10,10);
	glPopMatrix();

	gluDeleteQuadric(glpQ);
	*/

	/////////////////////////////
	
	/////////////////////////////
	// Draw the construction

	// stop and reset animation if animated too long
	// just a non-perfect security measure
	if(m_boltOffset+m_nutOffset>5.0f)
	{
		aniBolt=FALSE;
		aniNut=FALSE;
		m_boltOffset=0.0f;
		m_nutOffset=0.0f;
	}
	
	// draw the bolt
	// set material for bolt
	CMaterial::SetMaterial(SILVER);
	glPushMatrix();
	glRotatef(-m_boltv,0.0f,0.0f,1.0f);
	glTranslatef(0.0f,0.0f,-m_boltOffset);
	DrawBolt(3.0f,1.0f,7.0f,2.0f);
	glPopMatrix();

	glTranslatef(0.0f,0.0f,-3.0f);
	
	// draw the nut
	// set material for nut
	CMaterial::SetMaterial(SILVER);
	glPushMatrix();
	glRotatef(m_nutv,0.0f,0.0f,1.0f);
	glTranslatef(0.0f,0.0f,m_nutOffset);
	DrawNut(3.0f,1.0f,2.0f);
	glPopMatrix();

	// End draw the construction
	/////////////////////////////


	// finnish drawing
	glFlush();
}

void CStdView::InitializeRContext()
{
	COGLView::InitializeRContext();
	// change backcolor from default gray
	glClearColor(1.0f, 1.0f, 1.0f, 0.0f);

}

void CStdView::SetPerspective(int cx, int cy)
{
    // We will work on the projection matrix,
	// so set matrix mode
	glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
	// set up your viewing volume
	gluPerspective(70.0f, (GLdouble)cx/cy, 0.1f, 70.0f);
	// reset matrix mode 
	glMatrixMode(GL_MODELVIEW);
}


void CStdView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// User has pressed left button.Make a note
	// of this and prepare for intuitive rotation of scene
	m_PLast=point;
	SetCapture();
	CView::OnLButtonDown(nFlags, point);
}

void CStdView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// free capture and terminate rotation
	ReleaseCapture();
	CView::OnLButtonUp(nFlags, point);
}

void CStdView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// if left button is down 
	if(nFlags && MK_LBUTTON)
	{	
		// and has moved a significant dist
		if((abs(m_PLast.x-point.x)>2)||(abs(m_PLast.y-point.y)>2))
		{
			// then we find an intuitive rot angle
			CRect R;
			GetClientRect(&R);
			
			m_yv+=90.0f*((GLfloat (point.x-m_PLast.x))/GLfloat (R.Width()));
			m_xv+=90.0f*((GLfloat (point.y-m_PLast.y))/GLfloat (R.Height()));
			
			// rememeber point and force immediate redraw 
			m_PLast=point;
			RedrawWindow();
		}
	}
}

////////// bolt and nut ///////////////////////


void CStdView::DrawBolt(float R,float r, float Len, float len)
{
	float d60=2*PI/6.0f; // 60 degs in radians
	float d6=d60/6.0f;	 // 6 degs in radians

	GLUquadricObj *glpQ;
	glpQ=gluNewQuadric();

	// draw a bolt centered in origo
	// standing, head up, on xy-plane

	// draw bolt as a cylinder on origo
	glShadeModel(GL_SMOOTH);
	gluCylinder(glpQ,r,r,Len-len,36,36);
	gluDisk(glpQ,0.0f,r,20,20);
	
	glPushMatrix();

	// draw head with 6 sides
	glTranslatef(0.0f,0.0f,Len-len);
	glShadeModel(GL_FLAT);

	/*
	// fast version with little control
	gluDisk(glpQ,0.0f,R,6,6);
	gluCylinder(glpQ,R,R,len,6,6);
	glTranslatef(0.0f,0.0f,len);
	gluDisk(glpQ,0.0f,R,6,6);
	// end fast version 
	*/

	///*
	// handmade version which in part can be used also for nut
	// the 6 sides
	glEnable(GL_NORMALIZE);
	float v=0.0f;
	glBegin(GL_QUAD_STRIP);
	do{
		glNormal3f(R*(float)cos(v-d60/2.0f),R*(float)sin(v-d60/2.0f),0.0f);
		glVertex3f(R*(float)cos(v),R*(float)sin(v),len);
		glVertex3f(R*(float)cos(v),R*(float)sin(v),0.0f);
		v+=d60;
	}while(v<2*PI+0.01);
	glEnd();

	// the bottom 
	glPushMatrix();
	glBegin(GL_TRIANGLE_FAN);
	glNormal3f(0.0f,0.0f,-1.0f);
	v=0.0f;
	do{
		glVertex3f(R*(float)cos(v),R*(float)sin(v),0.0f);
		v+=d60;
	}while(v<2*PI+0.01);
	glEnd();
	
	// the top
	glTranslatef(0.0f,0.0f,len);
	glBegin(GL_TRIANGLE_FAN);
	glNormal3f(0.0f,0.0f,1.0f);
	v=0.0f;
	do{
		glVertex3f(R*(float)cos(v),R*(float)sin(v),0.0f);
		v+=d60;
	}while(v<2*PI+0.01);
	glEnd();
	glPopMatrix();
	
	// end of handmade bolt head
	//*/
	
	glPopMatrix();
	// draw spiral
	CMaterial::SetMaterial(BLACK_PLASTIC);
	float nTurns=Len*TURNS_PR_UNITLENGTH;
	float t=0.0f;
	glLineWidth(4.0f);
	glBegin(GL_LINE_STRIP);
	do
	{
		glVertex3f(
				r*(float)cos(2*PI*t),
				r*(float)sin(2*PI*t),
				t*((Len-len)/nTurns));
		t+=1.0f/36.0f;
	} while (t<nTurns);
	glEnd();

	gluDeleteQuadric(glpQ);
}


// answer to assigment 1b
void CStdView::DrawNut(float R,float r, float len)
{
	// Draw a 6-sided nut centered in origo,lying on xy-plane
	// Problem  is that we cannot draw concave polygons,
	// and we have no means to "subtract" holes.
	// We will therefore try to build the nut with
	// fanning polygons from the inner circle to the outer
	// A six-point circle in the center is not visually a
	// good solution, so we must do better
	// We do a 36 point circle in and out and we use clip-planes to
	// control outer boundary
	float d60=2*PI/6.0f; // 60 degs in radians
	float d6=d60/6.0f;	 // 6 degs in radians

	// do outline, as for bolt
	glShadeModel(GL_FLAT);
	glEnable(GL_NORMALIZE);
	float v=0.0f;
	glBegin(GL_QUAD_STRIP);
	do{
		glNormal3f(R*(float)cos(v-d60/2.0f),R*(float)sin(v-d60/2.0f),0.0f);
		glVertex3f(R*(float)cos(v),R*(float)sin(v),len);
		glVertex3f(R*(float)cos(v),R*(float)sin(v),0.0f);
		v+=d60;
	}while(v<2*PI+0.01);
	glEnd();

	// do bottom and top
	for(int ix=0;ix<6;ix++)
	{
		// set up clip-plane
		// Ax+By+Cz+D
		// c[]={A,B,C,D}
		// where  D=Ax+By+Cz
		// x,y,z any point on plane
		double c[4]={
				-(float)cos(d60*ix+d60/2.0f),
				-(float)sin(d60*ix+d60/2.0f),
				0.0,
				(float)cos(d60*ix+d60/2.0f)*R*(float)cos(d60*ix)
					+(float)sin(d60*ix+d60/2.0f)*R*(float)sin(d60*ix)
					};
		glClipPlane(GL_CLIP_PLANE1,c);
		glEnable(GL_CLIP_PLANE1);


		// do six sectors on bottom with this clip-plane
		glPushMatrix();
		for(int planes=0;planes<2;planes++)
		{
			glBegin(GL_QUAD_STRIP);
			for(int fix=0;fix<=6;fix++)
			{
					if(planes==0)
						glNormal3f(0.0f,0.0f,-1.0f);
					else
						glNormal3f(0.0f,0.0f,1.0f);
					glVertex3f(
							r*(float)cos(ix*d60+fix*d6),
							r*(float)sin(ix*d60+fix*d6),
							0.0f);
					glVertex3f(
							R*(float)cos(ix*d60+fix*d6),
							R*(float)sin(ix*d60+fix*d6),
							0.0f);

			}
			glEnd();
			// prepare for top
			glTranslatef(0.0f,0.0f,len);
		}
	glPopMatrix();
	glDisable(GL_CLIP_PLANE1);
	}
	glShadeModel(GL_SMOOTH);

	// do hole
	GLUquadricObj *glpQ;
	glpQ=gluNewQuadric();
	gluCylinder(glpQ,r,r,len,36,36);

	// do spiral
	CMaterial::SetMaterial(BLACK_PLASTIC);
	float nTurns=len*TURNS_PR_UNITLENGTH;
	float t=0.0f;
	glLineWidth(4.0f);
	glBegin(GL_LINE_STRIP);
	do
	{
		glVertex3f(
				(r)*(float)cos(2*PI*t),
				(r)*(float)sin(2*PI*t),
				t*(len/nTurns));
		t+=1.0f/36.0f;
	} while (t<nTurns);
	glEnd();
	gluDeleteQuadric(glpQ);
}



// answer to assigment 1b

void CStdView::OnAnimBolt() 
{
	aniBolt=!aniBolt;
	if((!aniBolt)&&(!aniNut))
		KillTimer(MY_TIMER);
	if(aniBolt)
		SetTimer(MY_TIMER,50,NULL);
}

void CStdView::OnAnimNut() 
{
	aniNut=!aniNut;
	if((!aniBolt)&&(!aniNut))
		KillTimer(MY_TIMER);
	if(aniNut)
		SetTimer(MY_TIMER,50,NULL);
}

void CStdView::OnUpdateAnimBolt(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(aniBolt);
}

void CStdView::OnUpdateAnimNut(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(aniNut);
}

void CStdView::OnTimer(UINT nIDEvent) 
{
	
	float dv=12.0f;
	if(aniBolt)
	{
		m_boltv+=dv;
		m_boltOffset+=(dv/360.0f)/TURNS_PR_UNITLENGTH;
	}
	if(aniNut)
	{
		m_nutv+=dv;
		m_nutOffset+=(dv/360.0f)/TURNS_PR_UNITLENGTH;
	}
	RedrawWindow();
	COGLView::OnTimer(nIDEvent);
}

void CStdView::OnAnimReset() 
{
	m_nutOffset=m_boltOffset=0.0f;
	RedrawWindow();
}
